<?php if(!defined('APPLICATION')) exit();
$PluginInfo['Filterbyprefix'] = array(
    'Name' => 'Filterbyprefix',
	'Description' => "Filterbyprefix - A Vanilla Forums plugin in that filters discussions based on prefixes specified in the URL.",
    'Version' => '1.3',
    'RequiredApplications' => array('Vanilla' => '2.6.4'),        /*This is what it was tested on...*/
	'RequiredPlugins' => array('PrefixDiscussion' => '1.4'),      /*Important                       */
    'RequiredTheme' => false,
	'SettingsPermission' => 'Garden.Settings.Manage',
	'RegisterPermissions' => array('Plugins.Filterbyprefix.View'),  /*Permission to filter on prefix*/
	'Author' => 'Ron Brahmson based on Zero Comments plugin by Zachary Doll',
    'MobileFriendly' => true,
    'HasLocale' => true,
    'License' => 'GPLv3'
);
/*
Important: This plugin requires the PrefixDiscussion plugin by R_J.
 
This plugin filters the discussion list based on the value of the prefix on discussion entries. The prefix has to be set via the PrefixDiscussion plugin.
   
Filtering is managed via the &filter= parameter added to the URL.  This may look weird at first, but consider that on our non-public forum we changed the discussion list to have a link on the prefix within the discussion list.  Thus a user clicking on the prefix will get a filtered list based on the clicked prefix.
   
There is a permission that need to be set to allows users to use filtered views.
   
Specifying prefix parameters:

a-forum/discussions/Filterbyprefix/&filter=Test  	- Only display discussions with the "Test" prefix

a-forum/discussions/Filterbyprefix/&filter=*  		- To filter by any prefix (hiding discussions without a prefix)
   
a-forum/discussions/Filterbyprefix/&filter=-  		- To filter by no prefix (hiding discussions with a prefix)
   

Note: If you peek at the source you will see that there is also a &C= filter parameter for category id (which filters on both category and prefix).

Attributions & Thanks: I am not a programmer so I lifted hgtonight's Zero Comments plugin and modified it to suit my needs. I also owe hgtonight, R_J and others many thanks for assisting me through many pointers over my many questions in the Vanilla community forums.  I couldn't have done this without the community.

Change log:
1.1 	- Initial Release
1.2 	- Removed special handling of a reserved prefix ("Attention")
		- Improved performance of filtering on any prefix (by changing the SQL statement)

No warranty whatsoever is implied by releasing this plugin to the Vanilla community.
*/

class FilterbyprefixPlugin extends Gdn_Plugin {

  /* Whether or not we are in the Filtered view
   * @var type
   */
  private $CustomView = false;
  
  /* The custom view of the discussion list
   * @param type $Sender
   * @param type $Args
   */
  public function DiscussionsController_Filterbyprefix_Create($Sender, $Args = '') {
    if (CheckPermission('Garden.Settings.Manage')) {
        var_dump ($Args);
        //die(0);
    }
	$Filter = null;
	$Catfilter = null;
	$Page = 'Filterbyprefix/{Page}';
	if (CheckPermission('Plugins.Filterbyprefix.View')) {	
		if (isset($_GET['filter'])) $Filter = '&filter='.$_GET['filter'];	//If filters specified on the url, duplicate for subsequent pages
		if (isset($_GET)) $Catfilter = '&c='.$_GET['c'];
	}
	else {
		$Page = '  ';
		Gdn::Controller()->Title(T('Recent Discussions'));
	}
	$this->CustomView = true;
    $Sender->View = 'Index';

	$Sender->SetData('_PagerUrl', 'discussions/'.$Page.$Filter.$Catfilter);
    $Sender->Index(GetValue(0, $Args, 'p1'));
  }

  /**
   * Set the count to the cache value. This will use a more pages unless caching is enabled.
   * 
   * @param DiscussionsController $Sender
   */
  public function DiscussionsController_Render_Before($Sender) {
    if($this->CustomView) {
      $Sender->SetData('CountDiscussions', Gdn::Cache()->Get('Filterbyprefix-Count'));
    }
  } 
   
  /**
   * If we are in the custom view modify the discussion model.
   * @param type $Sender
   */
  public function DiscussionModel_BeforeGet_Handler($Sender) {
	if($this->CustomView != true)  return;
	
	if (!CheckPermission('Plugins.Filterbyprefix.View')) {		//Validate that the user has permission to filtered views
        decho ($Filter);
        die(0);
		$Title = T('Unrecognized parameters:');                 //If not, don't accept the parameters
		foreach ($_GET as $key => $value) {
			$Title=$Title.$key."=".$value." ";
		}
		Gdn::Controller()->Title($Title);
		return;													// and return without any custom view
	}
	$Filter = null;												// Get the filter parameter from the URL
	if (array_key_exists('filter',$_GET)) $Filter = $_GET['filter'];;
	if ($Filter == null || $Filter == "") {
		return;  												//No filter parameter so no further filtering
	}
	$Catid = null;												//"Undocumented" filter by category id
	if (array_key_exists('c',$_GET)) $Catid = $_GET['c'];
	if ($Catid != null) { 	
		$Sender->SQL->Where('d.CategoryID', $Catid);			//Filter can be applied to a single numbered category	
	}
	if ($Filter == "*") {										//Filter on any prefix (exclude those without a prefix)
		$Sender->SQL->Where('d.Prefix <>',false);
		Gdn::Controller()->Title(T('Discussions marked with any prefix'));;
	}
	elseif ($Filter == "-") {									//Filter on no prefix
		//echo "<BR>1.2b filtering on no prefix ".$Filter;
		$Sender->SQL->Where('d.Prefix',null);					//Apply the filter to the prefix
		Gdn::Controller()->Title(T('Discussions not marked with a prefix'));
	}
	elseif ($Filter != null) {									//Filter on a specific prefix
		//echo "<BR>1.2c filtering on prefix ".$Filter;
		$Sender->SQL->Where('d.Prefix',$Filter);				//Apply the filter to the prefix
		Gdn::Controller()->Title(T('Discussions marked with ').'"'.$Filter.'"');
	}
  }
}
